@@ -322,10 +322,16 @@ int pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char **argv)
 
   for(i=0; i < argc; i++) {
      {
-	 char *tmp = xstrdup(argv[i]);
-	 strncpy(argbuf,strtok(tmp ,"="),255);
-	 strncpy(argval,strtok(NULL,"="),255);
-	 free(tmp);
+	 char *tmp = strchr(argv[i], '=');
+	 if(tmp) {
+	   strncpy(argbuf, argv[i], tmp-argv[i]);
+	   argbuf[tmp-argv[i]] = '\0';
+	   strncpy(argval, tmp+1, 255);
+	 }
+	 else {
+	   strncpy(argbuf, argv[i], 255);
+	   *argval = '\0';
+	 }
      }
 
      /* For PC functionality use "strict" -- historically "fascist" */
