@@ -457,7 +457,7 @@ main(argc, argv)
 		preauth_flag++;
 	}
 #ifdef RLOGIN
-	if (rflg && rlogin(host, name, sizeof name, term, sizeof term))
+	if (rflg && r_login(host, name, sizeof name, term, sizeof term))
 		preauth_flag++;
 #endif
 
@@ -638,7 +638,7 @@ top:
 		if (preauth_flag)
 			goto auth_ok;
 
-#ifdef 1
+#if 1
 		/*
 		 * Hack for passwordless accounts - not authenticated, but
 		 * if login is denied for some reason (securetty etc.), the
@@ -971,8 +971,8 @@ auth_ok:
 #endif
 	setup_limits(&pwent);  /* nice, ulimit etc. */
 	chown_tty(tty, &pwent);
-	setup_uid_gid(&pwent, is_console);
-
+	if(setup_uid_gid(&pwent, is_console))
+		exit(1);
 #ifdef KERBEROS
 	if (clear_pass)
 		login_kerberos(name, clear_pass);
