@@ -277,7 +277,7 @@ setlevel(argc, argv)
 	char *argv[];
 {
 	register struct levels *p;
-	int comret;
+	int comret, oldlevel;
 
 	if (argc > 2) {
 		char *sep;
@@ -313,10 +313,12 @@ setlevel(argc, argv)
 		return;
 	}
 	/* Start with a PBSZ of 1 meg */
+	oldlevel=level;
+	level=p->p_level;
 	if (p->p_level != PROT_C) setpbsz(1<<10);
 	comret = command("PROT %s", p->p_mode);
-	if (comret == COMPLETE)
-		level = p->p_level;
+	if (comret != COMPLETE)
+		level = oldlevel;
 }
 
 char *plevel[] = {
