@@ -525,7 +525,7 @@ getreply(expecteof)
 	sigtype cmdabort();
 	char ibuf[FTP_BUFSIZ], obuf[FTP_BUFSIZ];
 	int safe = 0;
-	extern char *strpbrk(), *strstr();
+	/*extern char *strpbrk(), *strstr();*/
 
 	ibuf[0] = '\0';
 	if (reply_parse) reply_ptr = reply_buf;
