@@ -1794,7 +1794,7 @@ void
 myoob()
 {
 	char *cp, *cs;
-	extern char *strpbrk();
+	/*extern char *strpbrk();*/
 
 	/* only process if transfer occurring */
 	if (!transflag)
@@ -2251,7 +2251,7 @@ send_file_list(whichfiles)
 	FILE *dout = NULL;
 	register char **dirlist, *dirname;
 	int simple = 0;
-	char *strpbrk();
+	/*char *strpbrk();*/
 	int ret = 0;
 
 	if (strpbrk(whichfiles, "~{[*?") != NULL) {
