@@ -1,23 +1,42 @@
 /*
- * Copyright (c) 1997,1998 Stanford University
+ * Copyright (c) 1997-1999  The Stanford SRP Authentication Project
+ * All Rights Reserved.
  *
- * The use of this software for revenue-generating purposes may require a
- * license from the owners of the underlying intellectual property.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * Within that constraint, permission to use, copy, modify, and distribute
- * this software and its documentation for any purpose is hereby granted
- * without fee, provided that the above copyright notices and this permission
- * notice appear in all copies of the software and related documentation.
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
  *
- * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
- * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
+ * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
+ * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
+ * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
  *
  * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
  * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
  * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
  * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * In addition, the following conditions apply:
+ *
+ * 1. Any software that incorporates the SRP authentication technology
+ *    must display the following acknowlegment:
+ *    "This product uses the 'Secure Remote Password' cryptographic
+ *     authentication system developed by Tom Wu (tjw@CS.Stanford.EDU)."
+ *
+ * 2. Any software that incorporates all or part of the SRP distribution
+ *    itself must also display the following acknowledgment:
+ *    "This product includes software developed by Tom Wu and Eugene
+ *     Jhong for the SRP Distribution (http://srp.stanford.edu/srp/)."
+ *
+ * 3. Redistributions in source or binary form must retain an intact copy
+ *    of this copyright notice and list of conditions.
  */
 
 #ifdef SRP
@@ -131,7 +150,7 @@ int
 srp_send(ap)
      Authenticator *ap;
 {
-  printf("[ Trying SRP ... ]\n");
+  printf("[ Trying SRP ... ]\r\n");
   if (!UserNameRequested) {
     if (auth_debug_mode) {
       printf("SRP: no user name supplied\r\n");
@@ -183,6 +202,7 @@ srp_is(ap, data, cnt)
 #ifdef	ENCRYPTION
   Session_Key skey;
 #endif
+  unsigned char type_check[2];
 
   if(cnt-- < 1)
     return;
@@ -271,6 +291,12 @@ srp_is(ap, data, cnt)
     /* Wait until now to send B, since it contains the key to "u" */
     Data(ap, SRP_CHALLENGE, B->data, B->len);
     B = NULL;
+
+    if ( ap->way & AUTH_ENCRYPT_MASK ) {
+      type_check[0] = (unsigned char) ap->type;
+      type_check[1] = (unsigned char) ap->way;
+      t_serveraddexdata(ts,type_check,2);
+    }
  
     A.data = data;
     A.len = cnt;
@@ -347,6 +373,7 @@ srp_reply(ap, data, cnt)
 #ifdef	ENCRYPTION
   Session_Key skey;
 #endif
+  unsigned char type_check[2];
 
   if(cnt-- < 1)
     return;
@@ -367,7 +394,7 @@ srp_reply(ap, data, cnt)
     }
 
     if(t_clientverify(tc, data) == 0) {
-      printf("[ SRP authentication successful ]\n");
+      printf("[ SRP authentication successful ]\r\n");
 
 #ifdef	ENCRYPTION
       skey.type = SK_GENERIC;
@@ -377,7 +404,7 @@ srp_reply(ap, data, cnt)
 #endif
     }
     else
-      printf("[ Warning: SRP server authentication failed ]\n");
+      printf("[ Warning: SRP server authentication failed ]\r\n");
     break;
   case SRP_PARAMS:
     if(!UserNameRequested) {
@@ -398,7 +425,7 @@ srp_reply(ap, data, cnt)
     data += n.len;
     cnt -= n.len;
 
-    printf("[ Using %d-bit modulus for '%s' ]\n", 8 * n.len, UserNameRequested);
+    printf("[ Using %d-bit modulus for '%s' ]\r\n", 8 * n.len, UserNameRequested);
 
     g.len = decode_length(data);
     data += 2;
@@ -443,6 +470,12 @@ srp_reply(ap, data, cnt)
       break;
     }
       
+    if ( ap->way & AUTH_ENCRYPT_MASK ) {
+      type_check[0] = (unsigned char) ap->type;
+      type_check[1] = (unsigned char) ap->way;
+      t_clientaddexdata(tc,type_check,2);
+    }
+
     B.data = data;
     B.len = cnt;
     t_clientgetkey(tc, &B);
@@ -454,7 +487,7 @@ srp_reply(ap, data, cnt)
 
   default:
     if(auth_debug_mode)
-      printf("Unknown reply option\n");
+      printf("Unknown reply option\r\n");
     break;
   }
 }
@@ -472,6 +505,7 @@ srp_status(ap, name, level)
 	  strcpy(name, UserNameRequested);
 	  return AUTH_VALID;
 	}
+	return AUTH_USER;
 }
 
 #define	BUMP(buf, len)		while (*(buf)) {++(buf), --(len);}
