@@ -70,6 +70,7 @@ static char sccsid[] = "@(#)commands.c	8.4 (Berkeley) 5/30/95";
 #include <pwd.h>
 #include <varargs.h>
 #include <errno.h>
+#include <string.h>
 
 #include <arpa/telnet.h>
 
@@ -94,7 +95,7 @@ static char sccsid[] = "@(#)commands.c	8.4 (Berkeley) 5/30/95";
 #endif /* INET6 */
 
 #ifndef	MAXHOSTNAMELEN
-#define	MAXHOSTNAMELEN 64
+#define	MAXHOSTNAMELEN 256 /* per rfc */
 #endif	MAXHOSTNAMELEN
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
@@ -103,6 +104,7 @@ int tos = -1;
 
 char	*hostname;
 static char _hostname[MAXHOSTNAMELEN];
+struct in_addr hostaddr;
 
 extern char *getenv();
 
@@ -1106,6 +1108,7 @@ dokludgemode()
     send_wont(TELOPT_LINEMODE, 1);
     send_dont(TELOPT_SGA, 1);
     send_dont(TELOPT_ECHO, 1);
+    return 1;
 }
 #endif
 
@@ -1707,7 +1710,7 @@ env_init()
 	extern char **environ;
 	register char **epp, *cp;
 	register struct env_lst *ep;
-	extern char *strchr();
+	/*extern char *strchr();*/
 
 	for (epp = environ; *epp; epp++) {
 		if (cp = strchr(*epp, '=')) {
@@ -1859,7 +1862,7 @@ env_default(init, welldefined)
 
 	if (init) {
 		nep = &envlisthead;
-		return;
+		return NULL;
 	}
 	if (nep) {
 		while (nep = nep->next) {
@@ -1881,6 +1884,16 @@ env_getvalue(var)
 	return(NULL);
 }
 
+        int 
+env_is_exported(var) unsigned char * var;
+{
+        register struct env_lst *ep;
+        
+        if (ep = env_find(var))
+            return(ep->export);
+        return 0;
+}
+
 #if defined(OLD_ENVIRON) && defined(ENV_HACK)
 	void
 env_varval(what)
@@ -2164,8 +2177,9 @@ status(argc, argv)
     int	 argc;
     char *argv[];
 {
+    extern char *inet_ntoa();
     if (connected) {
-	printf("Connected to %s.\n", hostname);
+	printf("Connected to %s (%s).\n", hostname,inet_ntoa(hostaddr));
 	if ((argc < 2) || strcmp(argv[1], "notmuch")) {
 	    int mode = getconnmode();
 
@@ -2240,6 +2254,10 @@ ayt_status()
 
 unsigned long inet_addr();
 
+#ifndef INADDR_NONE
+#define INADDR_NONE ((unsigned long) 0xffffffff)
+#endif
+
     int
 tn(argc, argv)
     int argc;
@@ -2376,7 +2394,7 @@ tn(argc, argv)
     } else {
 #endif
 	temp = inet_addr(hostp);
-	if (temp != (unsigned long) -1) {
+	if (temp != INADDR_NONE) {
 	    sin.sin_addr.s_addr = temp;
 	    sin.sin_family = AF_INET;
 	    (void) strcpy(_hostname, hostp);
@@ -2403,6 +2421,7 @@ tn(argc, argv)
 #if	defined(IP_OPTIONS) && defined(IPPROTO_IP)
     }
 #endif
+    hostaddr.s_addr = sin.sin_addr.s_addr;
     if (portp) {
 	if (*portp == '-') {
 	    portp++;
@@ -2486,6 +2505,8 @@ tn(argc, argv)
 		host->h_addr_list++;
 		memmove((caddr_t)&sin.sin_addr,
 			host->h_addr_list[0], host->h_length);
+		memmove((caddr_t)&hostaddr,
+			host->h_addr_list[0], host->h_length);
 		(void) NetClose(net);
 		continue;
 	    }
@@ -2494,12 +2515,26 @@ tn(argc, argv)
 	    return 0;
 	}
 	connected++;
+        host = gethostbyaddr((char *) &sin.sin_addr, sizeof(struct in_addr),
+                              sin.sin_family);
+        if ( host ) {
+            strncpy(_hostname, host->h_name,sizeof(_hostname));
+            hostname = _hostname;
+        }
 #if	defined(AUTHENTICATION) || defined(ENCRYPTION)
 	auth_encrypt_connect(connected);
 #endif	/* defined(AUTHENTICATION) || defined(ENCRYPTION) */
     } while (connected == 0);
 #endif /* INET6 */
+    if ( user )
+        user = strdup(user);
+    if ( hostp )
+        hostp = strdup(hostp);
     cmdrc(hostp, hostname);
+    if ( hostp ) {
+        free(hostp);
+        hostp = NULL;
+    }
     if (autologin && user == NULL) {
 	struct passwd *pw;
 
@@ -2511,6 +2546,8 @@ tn(argc, argv)
 		else
 			user = NULL;
 	}
+        if ( user )
+            user = strdup(user);
     }
     if (user) {
 	env_define((unsigned char *)"USER", (unsigned char *)user);
@@ -2519,6 +2556,10 @@ tn(argc, argv)
     (void) call(status, "status", "notmuch", 0);
     if (setjmp(peerdied) == 0)
 	telnet(user);
+    if ( user ) {
+        free(user);
+        user = NULL;
+    }
     (void) NetClose(net);
     ExitString("Connection closed by foreign host.\n",1);
     /*NOTREACHED*/
@@ -2965,7 +3006,7 @@ sourceroute(arg, cpp, lenp)
 		if (!c)
 			cp2 = 0;
 
-		if ((tmp = inet_addr(cp)) != -1) {
+		if ((tmp = inet_addr(cp)) != INADDR_NONE) {
 			sin_addr.s_addr = tmp;
 		} else if (host = gethostbyname(cp)) {
 #if	defined(h_addr)
