@@ -869,11 +869,18 @@ deadpeer(sig)
 	longjmp(peerdied, -1);
 }
 
+int intr_happened = 0;
+int intr_waiting = 0;
+
     /* ARGSUSED */
     SIG_FUNC_RET
 intr(sig)
     int sig;
 {
+    if ( intr_waiting ) {
+        intr_happened = 1;
+        return;
+    }
     if (localchars) {
 	intp();
 	return;
