@@ -670,6 +670,16 @@ usage()
 	exit(1);
 }
 
+static void encrypt_failure()
+{
+    char *error_message =
+	"Encryption was not successfully negotiated.  Goodbye.\r\n\r\n";
+
+    writenet(error_message, strlen(error_message));
+    netflush();
+    exit(1);
+}
+
 /*
  * getterminaltype
  *
@@ -702,6 +712,7 @@ getterminaltype(name)
 
 #ifdef	ENCRYPTION
     send_will(TELOPT_ENCRYPT, 1);
+    send_do(TELOPT_ENCRYPT, 1);
 #endif	/* ENCRYPTION */
     send_do(TELOPT_TTYPE, 1);
     send_do(TELOPT_TSPEED, 1);
@@ -711,6 +722,7 @@ getterminaltype(name)
     while (
 #ifdef	ENCRYPTION
 	   his_do_dont_is_changing(TELOPT_ENCRYPT) ||
+           his_will_wont_is_changing(TELOPT_ENCRYPT) ||
 #endif	/* ENCRYPTION */
 	   his_will_wont_is_changing(TELOPT_TTYPE) ||
 	   his_will_wont_is_changing(TELOPT_TSPEED) ||
@@ -727,7 +739,27 @@ getterminaltype(name)
     if (his_state_is_will(TELOPT_ENCRYPT)) {
 	encrypt_wait();
     }
+    if (auth_must_encrypt()) {
+	time_t timeout = time(0) + 60;
+	
+	if (my_state_is_dont(TELOPT_ENCRYPT) ||
+	    my_state_is_wont(TELOPT_ENCRYPT))
+	    encrypt_failure();
+
+	if (!EncryptStartInput() || !EncryptStartOutput())
+	    encrypt_failure();
+
+	while (!encrypt_is_encrypting()) {
+	    if (time(0) > timeout)
+		encrypt_failure();
+	    ttloop();
+	}
+    }
 #endif	/* ENCRYPTION */
+    /* Options like environment require authentication and encryption
+    negotiation to be completed.*/
+    auth_negotiated = 1;
+
     if (his_state_is_will(TELOPT_TSPEED)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
@@ -795,11 +827,13 @@ getterminaltype(name)
 	 */
 	if (his_state_is_will(TELOPT_TTYPE) && !terminaltypeok(terminaltype)) {
 	    (void) strncpy(first, terminaltype, sizeof(first));
+            first[sizeof(first) - 1] = '\0';
 	    for(;;) {
 		/*
 		 * Save the unknown name, and request the next name.
 		 */
 		(void) strncpy(last, terminaltype, sizeof(last));
+                last[sizeof(last) - 1] = '\0';
 		_gettermname();
 		if (terminaltypeok(terminaltype))
 		    break;
@@ -819,6 +853,7 @@ getterminaltype(name)
 		     _gettermname();
 		    if (strncmp(first, terminaltype, sizeof(first)) != 0)
 			(void) strncpy(terminaltype, first, sizeof(first));
+                    terminaltype[sizeof(terminaltype) - 1] = '\0';
 		    break;
 		}
 	    }
@@ -852,7 +887,7 @@ terminaltypeok(s)
 {
     char buf[1024];
 
-    if (terminaltype == NULL)
+    if (!s || !*s)
 	return(1);
 
     /*
@@ -868,7 +903,7 @@ terminaltypeok(s)
 }
 
 #ifndef	MAXHOSTNAMELEN
-#define	MAXHOSTNAMELEN 64
+#define	MAXHOSTNAMELEN 256 /* per rfc */
 #endif	/* MAXHOSTNAMELEN */
 
 char *hostname;
@@ -986,7 +1021,7 @@ doit(who)
 	 */
 	*user_name = 0;
 	level = getterminaltype(user_name);
-	setenv("TERM", terminaltype ? terminaltype : "network", 1);
+	setenv("TERM", (terminaltype && *terminaltype) ? terminaltype : "network", 1);
 #ifdef INET6
 	inet_ntop(AF_INET6, &((struct sockaddr_in6 *)who)->sin6_addr, remote_host_addr, sizeof(remote_host_addr));
 	setenv("REMOTE_IPADDRESS", remote_host_addr, 0);
